@@ -1,9 +1,9 @@
 """System Bridge Shared: WebSocket Client"""
 from __future__ import annotations
 
+from collections.abc import Callable
 import json
 import socket
-from collections.abc import Callable
 
 import aiohttp
 
@@ -32,9 +32,6 @@ from systembridgeconnector.exceptions import (
 class WebSocketClient(Base):
     """WebSocket Client"""
 
-    _session: aiohttp.ClientSession
-    _websocket: aiohttp.ClientWebSocketResponse
-
     def __init__(
         self,
         api_host: str,
@@ -45,7 +42,9 @@ class WebSocketClient(Base):
         super().__init__()
         self._api_host = api_host
         self._api_port = api_port
-        self._api_key = api_key
+        self._api_key = api_key  #
+        self._session: aiohttp.ClientSession | None = None
+        self._websocket: aiohttp.ClientWebSocketResponse | None = None
 
     @property
     def connected(self) -> bool:
@@ -65,7 +64,9 @@ class WebSocketClient(Base):
         session: aiohttp.ClientSession | None = None,
     ) -> None:
         """Connect to server"""
-        if not session:
+        if session:
+            self._session = session
+        else:
             self._session = aiohttp.ClientSession()
         url = f"ws://{self._api_host}:{self._api_port}/api/websocket"
         self._logger.info("Connecting to WebSocket: %s", url)
@@ -86,123 +87,162 @@ class WebSocketClient(Base):
 
     async def exit_backend(self) -> None:
         """Exit backend"""
+        if not self.connected:
+            raise ConnectionClosedException("Connection is closed")
+
         self._logger.info("Exiting backend")
-        await self._websocket.send_str(
-            json.dumps(
-                {
-                    "event": TYPE_EXIT_APPLICATION,
-                    "api-key": self._api_key,
-                }
+        if self._websocket is not None:
+            await self._websocket.send_str(
+                json.dumps(
+                    {
+                        "event": TYPE_EXIT_APPLICATION,
+                        "api-key": self._api_key,
+                    }
+                )
             )
-        )
 
     async def get_data(
         self,
         modules: list[str],
     ) -> None:
         """Get data from server"""
+        if not self.connected:
+            raise ConnectionClosedException("Connection is closed")
+
         self._logger.info("Getting data from server: %s", modules)
-        await self._websocket.send_str(
-            json.dumps(
-                {
-                    "event": TYPE_GET_DATA,
-                    "api-key": self._api_key,
-                    "modules": modules,
-                }
+
+        if self._websocket is not None:
+            await self._websocket.send_str(
+                json.dumps(
+                    {
+                        "event": TYPE_GET_DATA,
+                        "api-key": self._api_key,
+                        "modules": modules,
+                    }
+                )
             )
-        )
 
     async def register_data_listener(
         self,
         modules: list[str],
     ) -> None:
         """Register data listener"""
+        if not self.connected:
+            raise ConnectionClosedException("Connection is closed")
+
         self._logger.info("Registering data listener: %s", modules)
-        await self._websocket.send_str(
-            json.dumps(
-                {
-                    "event": TYPE_REGISTER_DATA_LISTENER,
-                    "api-key": self._api_key,
-                    "modules": modules,
-                }
+        if self._websocket is not None:
+            await self._websocket.send_str(
+                json.dumps(
+                    {
+                        "event": TYPE_REGISTER_DATA_LISTENER,
+                        "api-key": self._api_key,
+                        "modules": modules,
+                    }
+                )
             )
-        )
 
     async def keyboard_keypress(
         self,
         key: str,
     ) -> None:
         """Keyboard keypress"""
+        if not self.connected:
+            raise ConnectionClosedException("Connection is closed")
+
         self._logger.info("Press key: %s", key)
-        await self._websocket.send_str(
-            json.dumps(
-                {
-                    "event": TYPE_KEYBOARD_KEYPRESS,
-                    "api-key": self._api_key,
-                    "key": key,
-                }
+        if self._websocket is not None:
+            await self._websocket.send_str(
+                json.dumps(
+                    {
+                        "event": TYPE_KEYBOARD_KEYPRESS,
+                        "api-key": self._api_key,
+                        "key": key,
+                    }
+                )
             )
-        )
 
     async def keyboard_text(
         self,
         text: str,
     ) -> None:
         """Keyboard keypress"""
+        if not self.connected:
+            raise ConnectionClosedException("Connection is closed")
+
         self._logger.info("Enter text: %s", text)
-        await self._websocket.send_str(
-            json.dumps(
-                {
-                    "event": TYPE_KEYBOARD_TEXT,
-                    "api-key": self._api_key,
-                    "text": text,
-                }
+        if self._websocket is not None:
+            await self._websocket.send_str(
+                json.dumps(
+                    {
+                        "event": TYPE_KEYBOARD_TEXT,
+                        "api-key": self._api_key,
+                        "text": text,
+                    }
+                )
             )
-        )
 
     async def open_path(
         self,
         path: str,
     ) -> None:
         """Open path"""
+        if not self.connected:
+            raise ConnectionClosedException("Connection is closed")
+
         self._logger.info("Opening path: %s", path)
-        await self._websocket.send_str(
-            json.dumps(
-                {
-                    "event": TYPE_OPEN,
-                    "api-key": self._api_key,
-                    "url": path,
-                }
+        if self._websocket is not None:
+            await self._websocket.send_str(
+                json.dumps(
+                    {
+                        "event": TYPE_OPEN,
+                        "api-key": self._api_key,
+                        "url": path,
+                    }
+                )
             )
-        )
 
     async def open_url(
         self,
         url: str,
     ) -> None:
         """Open url"""
+        if not self.connected:
+            raise ConnectionClosedException("Connection is closed")
+
         self._logger.info("Opening URL: %s", url)
-        await self._websocket.send_str(
-            json.dumps(
-                {
-                    "event": TYPE_OPEN,
-                    "api-key": self._api_key,
-                    "url": url,
-                }
+        if self._websocket is not None:
+            await self._websocket.send_str(
+                json.dumps(
+                    {
+                        "event": TYPE_OPEN,
+                        "api-key": self._api_key,
+                        "url": url,
+                    }
+                )
             )
-        )
 
     async def listen_for_messages(
         self,
         callback: Callable,
     ) -> None:
         """Listen for messages"""
-        self._logger.info("Listen for messages")
-        while not self._websocket.closed:
-            await callback(await self.receive_message())
+        if not self.connected:
+            raise ConnectionClosedException("Connection is closed")
 
-    async def receive_message(self) -> dict:
+        self._logger.info("Listen for messages")
+        if self._websocket is not None:
+            while not self._websocket.closed:
+                await callback(await self.receive_message())
+
+    async def receive_message(self) -> dict | None:
         """Receive message"""
+        if not self.connected:
+            raise ConnectionClosedException("Connection is closed")
+
+        if self._websocket is None:
+            return None
+
         message = await self._websocket.receive()
 
         if message.type == aiohttp.WSMsgType.ERROR:
